@@ -1,3 +1,4 @@
+from __future__ import print_function
 import sys
 import Pyro4
 from excep import MyError
@@ -34,7 +35,7 @@ try:
 except MyError:
     print("MYERROR: %s" % sys.exc_info()[1])
 
-print("*** invoking server method that crashes, catching traceback ***")
+print("\n*** invoking server method that crashes, catching traceback ***")
 try:
     print(test.complexerror())
 except Exception:
@@ -43,5 +44,8 @@ except Exception:
     print("".join(Pyro4.util.getPyroTraceback()))
     print("<<<<<<< end of Pyro traceback")
 
+print("\n*** installing pyro's excepthook")
+sys.excepthook=Pyro4.util.excepthook
 print("*** invoking server method that crashes, not catching anything ***")
-print(test.complexerror())
+print(test.complexerror())     # due to the excepthook, the exception will show the pyro error
+
